-- Test add_product
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(14);

set search_path to auth, numerus, public;

select has_function('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]']);
select function_lang_is('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]'], 'plpgsql');
select function_returns('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]'], 'uuid');
select isnt_definer('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]']);
select volatility_is('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]'], 'volatile');
select function_privs_are('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]'], 'guest', array []::text[]);
select function_privs_are('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]'], 'invoicer', array ['EXECUTE']);
select function_privs_are('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]'], 'admin', array ['EXECUTE']);
select function_privs_are('numerus', 'add_product', array ['integer', 'text', 'text', 'text', 'integer[]', 'tag_name[]'], 'authenticator', array []::text[]);


set client_min_messages to warning;
truncate product_tax cascade;
truncate product cascade;
truncate tax cascade;
truncate tax_class cascade;
truncate payment_method cascade;
truncate company cascade;
reset client_min_messages;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (1, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 111)
     , (2, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 222)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (111, 1, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into tax_class (tax_class_id, company_id, name)
values (11, 1, 'tax')
     , (22, 2, 'tax')
;

insert into tax (tax_id, company_id, tax_class_id, name, rate)
values (3, 1, 11, 'IRPF -15 %', -0.15)
     , (4, 1, 11, 'IVA 21 %', 0.21)
     , (5, 2, 22, 'IRPF -7 %', -0.07)
     , (6, 2, 22, 'IVA 10 %', 0.10)
;


select lives_ok(
	$$ select add_product(1, 'Product 1', 'Description 1', '12.12', array[3, 4], array['tag1']) $$,
	'Should be able to add product to first company'
);

select lives_ok(
	$$ select add_product(2, 'Product 2', 'Description 2', '24.24', array[6], array['tag1', 'tag2']) $$,
	'Should be able to add product to second company'
);

select lives_ok(
	$$ select add_product(2, 'Product 3', 'Description 3', '36.36', array[]::integer[], array[]::tag_name[]) $$,
	'Should be able to add product without taxes'
);

select bag_eq(
	$$ select company_id, name, description, price, tags, created_at from product $$,
	$$ values (1, 'Product 1', 'Description 1', 1212, '{tag1}'::tag_name[], current_timestamp)
	        , (2, 'Product 2', 'Description 2', 2424, '{tag1,tag2}'::tag_name[], current_timestamp)
		, (2, 'Product 3', 'Description 3', 3636, '{}'::tag_name[], current_timestamp)
	$$,
	'Should have added all three products'
);

select bag_eq(
	$$ select tax_id, name from product_tax join product using (product_id) $$,
	$$ values (3, 'Product 1')
	        , (4, 'Product 1')
		, (6, 'Product 2')
	$$,
	'Should have added the taxes for the products we told to'
);


select *
from finish();

rollback;
